#!/bin/ksh
#set -vx

# Introscope Enterprise Manager ServiceNow alarm script.

# Enable "Notify by individual metrix" for the used alert to prevent 
# errors like "The parameter or environment lists are too long.". You will
# find some messages like "Action "ServiceNow Notification" failed to 
# execute shell command "./scripts/servnow.sh" in the log.

# The action Shell Command parameter is the way how to handle the alert and
# some of the following parameter "Pass Text to Shell" or
# "Alert Name", "Alert Status", "Alert Time" and "Agent Name(s)" or
# "Metric Name(s)". And additional "Previous Alert State" and
# "Threshold Value".
# Not used are "EM Name", "EM Host", "EM IP Address" and
# "Has Alert State Changed".

if [ -L $0 ]; then
  ls="`ls -l $0`"
  PRG="${ls##* -> }"
else
  PRG="$0"
fi

if [ "$PRG" = ^/ ]; then
  DIR=`dirname $PRG`
else
  DIR="`cd \`dirname $PRG\`; pwd -P`"
  PRG="`cd \`dirname $PRG\`; pwd -P`/`basename $PRG`"
fi

DESC="Introscope Enterprise Manager alarm script"
SCRIPT=${PRG##*/}
NAME=${SCRIPT%.*}

DIENST=`echo "$DIR" | cut -c6-12`
OS=`uname -s | tr 'A-Z' 'a-z'`

cd $DIR

# Monitor resource status used by ServiceNow.
UNKNOWN='unknown'
ONLINE='online'
OFFLINE='offline'
FAILED='failed'
STUCK='stuck'
#PENONLINE='penonline'
#PENOFFLINE='penoffline'
MIXED='mixed'
INELIGIBLE='ineligible'

# ServiceNow init.
SNCMD="./curl.sh"
SNLOG="../logs/servnow.log"
SNURL='https://servicenow.local/api/now/table'
SNLBL="INTROSCOPE"
SNTAB=""
SNMSG="INTR000I"
SNSTA="$UNKNOWN"
SNENV="./servnow.env"

SNDEBUG=''
SNUPDATE=''
SNUSER='now_user'
SNPASS='password'
if [ -f "$SNENV" ]; then
  . $SNENV

  if [ -n "$NOW_HOST" ]; then
    SNHOST="$NOW_HOST"
    SNURL="https://${NOW_HOST}/api/now/table"
  fi
  if [ -n "$NOW_USER" ]; then
    SNUSER="$NOW_USER"
  fi
  if [ -n "$NOW_PASS" ]; then
    SNPASS="$NOW_PASS"
  fi
  if [ -n "$NOW_DEBUG" ]; then
    SNDEBUG="$NOW_DEBUG"
  fi
  if [ -n "$NOW_UPDATE" ]; then
    SNUPDATE="$NOW_UPDATE"
  fi
fi

# ServiceNow return code handling.
RC=0
rc=0

# Handle the command parameters.
USERLBL=''
USERTAB=''
USERMSG=''
USERSTA=''
parm=`echo "$1" | cut -f 1 -d ' '`
if [ -n "$parm" ]; then
  SNLBL="$parm"
  USERLBL="$parm"
fi
parm=`echo "$1" | cut -f 2 -d ' '`
if [ -n "$parm" ]; then
  SNTAB="$parm"
  USERTAB="$parm"
fi
parm=`echo "$1" | cut -f 3 -d ' '`
if [ -n "$parm" ]; then
  SNMSG="$parm"
  USERMSG="$parm"
fi
parm=`echo "$1" | cut -f 4 -d ' '`
if [ -n "$parm" ]; then
  SNSTA="$parm"
  USERSTA="$parm"
fi
shift

if [ -z "$USER" ]; then
  USER=`whoami 2>/dev/null`
  if [ -z "$USER" ]; then
    USER="$LOGNAME"
  fi
fi
user=`echo "$USER" | tr 'a-z' 'A-Z'`

if [[ "$DIENST" = 'as'* ]]; then
  dienst="$DIENST"
else
  [ -n "$USER" ] && dienst="$USER" || dienst="$LOGNAME"
fi

if [ -z "$HOST" ]; then
  HOST=`hostname`
fi
host=`echo ${HOST%%.*} | tr 'a-z' 'A-Z'`

#echo "$*" >> ${SNLOG}

# Handle ServiceNow status information similar to the EMail alerts send by
# Introscope, but use the state value only.
# from: introscope@f-i.de
# subject: CA APM Alert from ${EM_Host}: ${Alert_Name} in ${Alert_State} state
# body: Alert Name: ${Alert_Name}
# Time Triggered: ${Alert_Time}
# Alert Status: ${Alert_State}
# Problem Details:
# ${Problem_Detail_List}

desc='Introscope alert.'

# Handle the text input line.
text="$*"
stat="$UNKNOWN"
alert_stat='unknown'

if [[ "$text" = *' reported:  The Alert '* ]]; then
# Used Command Parameters.
# Pass Text to Shell.
  echo "$text" | \
  awk '{
    if (index($0, "reported:  The Alert") != 0) {
      l=split($0, line, "         ")
      for(i=1; i<=l; i++) {
        if (i == 1) {
          h=split(line[i], head, ":  ")
          print head[1]
          print substr(head[2], 1, length(head[2])-1)
        } else {
          print line[i]
        }
      }
    }
  }' | \
  while read -r line; do
#    echo "$line" >> ${SNLOG} 
    if [[ "$line" = *'Introscope Enterprise Manager'* ]]; then
      time=`echo "$line" | cut -f 1-4 -d ' '`
    elif [[ "$line" = *'The Alert'*'opened a caution alert'* ]]; then
      alert=`echo ${line} | sed -n -e 's/.*The Alert \(.*\) opened a caution alert.*/\1/p'`
      stat="$OFFLINE"
      alert_stat='caution'
    elif [[ "$line" = *'The Alert'*'exceeded caution target'* ]]; then
      alert=`echo ${line} | sed -n -e 's/.*The Alert \(.*\) exceeded caution target.*/\1/p'`
      stat="$OFFLINE"
      alert_stat='caution'
    elif [[ "$line" = *'The Alert'*'opened a danger alert'* ]]; then
      alert=`echo ${line} | sed -n -e 's/.*The Alert \(.*\) opened a danger alert.*/\1/p'`
      stat="$OFFLINE"
      alert_stat='danger'
    elif [[ "$line" = *'The Alert'*'exceeded danger target'* ]]; then
      alert=`echo ${line} | sed -n -e 's/.*The Alert \(.*\) exceeded danger target.*/\1/p'`
      stat="$OFFLINE"
      alert_stat='danger'
    elif [[ "$line" = *'The Alert'*'closed a caution alert'* ]]; then
      alert=`echo ${line} | sed -n -e 's/.*The Alert \(.*\) closed a caution alert.*/\1/p'`
      stat="$ONLINE"
      alert_stat='normal'
    elif [[ "$line" = *'The Alert'*'closed a danger alert'* ]]; then
      alert=`echo ${line} | sed -n -e 's/.*The Alert \(.*\) closed a danger alert.*/\1/p'`
      stat="$ONLINE"
      alert_stat='normal'
    else
      host=`echo "$line" | cut -f 2 -d '|'`
      if [[ "$host" = 'Custom Metric Host'* ]]; then
        host=`hostname`
      fi

# Handle status notifications only.
      if [ "$SNLBL" = 'CONNECTIONSTATUS' ]; then
        table='u_middleware_instanz'
        host=`echo "$line" | cut -f 6 -d '|'`
        agent=`echo "$line" | cut -f 8 -d '|'`
        attr=`echo "$agent" | cut -f 2 -d ':' | cut -f 1 -d ' ' | tr 'a-z' 'A-Z'`
        value=`echo "$agent" | cut -f 2 -d ':' | cut -f 3 -d ' ' | tr 'a-z' 'A-Z'`
        agent=`echo "$agent" | cut -f 1 -d ':'`

        if [ "$SNLBL" = "$attr" ]; then
# Connection metrics have these values.
# No data=0, Connected=1, Slow report=2, Disconnected=3
          if [ "$value" -eq 1 -o "$value" -eq 2 ]; then
            stat="$ONLINE"
          else
            stat="$OFFLINE"
          fi
# Not a ConnectionStatus metric.
        else
          table=''
          echo "#### $line" >> ${SNLOG} 
        fi

# Map the used resource name.
# Agent "env.reg.ope.as12345.1" to resource "jvm.ope.as12345.1".
        name="jvm."`echo "$agent" | cut -f 3-5 -d '.'`
      else
        table=''
        agent=`echo "$line" | cut -f 4 -d '|'`

        echo "#### $line" >> ${SNLOG} 
      fi        

#      host=`echo ${host%%.*} | tr 'a-z' 'A-Z'`

#      text=`echo "APM ${agent} ${alert} in ${alert_stat} state" | sed -e 's/\&/\&amp;/g' \
#        -e 's/"//g' -e "s/'/\&apos;/g" -e 's/</\&lt;/g' -e 's/>/\&gt;/g'`
#      desc=`echo ${line} | sed -e 's/\&/\&amp;/g' -e 's/"/\&quot;/g' \
#        -e "s/'/\&apos;/g" -e 's/</\&lt;/g' -e 's/>/\&gt;/g'`

#      alert=`echo ${alert} | sed -e 's/\&/\&amp;/g' -e 's/"/\&quot;/g' \
#        -e "s/'/\&apos;/g" -e 's/</\&lt;/g' -e 's/>/\&gt;/g'`
      
      if [ -n "$table" ]; then  
        SNURL="https://${NOW_HOST}/api/now/table"
        table='u_middleware_instanz'

# Get the used sys_id.
# {"result":[{"sys_id":"ca83c3816f054b0027e0cc60cb1f4e8e"}]} or
# {"result":[{"sys_id":"996c61a16fff3280029ecc60cb1f4ed8","u_instanz_status":"up","u_monitor_status":"online"}]}
# {"result":[{"sys_id":"996c61a16fff3280029ecc60cb1f4ed8","u_monitor_status":"online"}]}
# {"result":[]}
        data='"'"${SNURL}/${table}"
        data=$data'?sysparm_query=name='"${name}"
        data=$data'&sysparm_fields=sys_id%2Cu_instanz_status%2Cu_monitor_status'
        data=$data'&sysparm_limit=1"'
        data=$data' --request GET'
        data=$data' --insecure --user "'${SNUSER}'":"'${SNPASS}'"'
        data=$data' --silent --show-error'

        if [ "$SNDEBUG" != 'curl' ]; then
          result=`${SNCMD} ${data} 2>/dev/null`
        else
          date >> ${SNLOG}
          echo ${SNCMD} ${data} >> ${SNLOG} 2>&1
          result=`${SNCMD} ${data} | tee -a ${SNLOG}`
          echo "" >> ${SNLOG}
        fi

#        sysid=`echo "$result" | sed -n -e 's/.*{"sys_id":"\(.*\)"}.*/\1/p'`
        sysid=`echo "$result" | sed -n -e 's/.*"sys_id":"\([^"]*\)".*/\1/p'`
        statinst=`echo "$result" | sed -n -e 's/.*"u_instanz_status":"\([^"]*\)".*/\1/p'`
        statmon=`echo "$result" | sed -n -e 's/.*"u_monitor_status":"\([^"]*\)".*/\1/p'`

# Status mapping.
        if [ "$stat" = "$ONLINE" ]; then
          if [ "$statinst" = 'control' ]; then
            :
          elif [ "$statinst" = 'down' ]; then
            stat="$STUCK"
          elif [ "$statinst" = 'shutdown' ]; then
            stat="$STUCK"
          elif [ "$statinst" = 'stopping' ]; then
            stat="$STUCK"
          else
            :
          fi
        elif [ "$stat" = "$OFFLINE" ]; then
          if [ "$statinst" = 'control' ]; then
            :
          elif [ "$statinst" = 'up' ]; then
            stat="$FAILED"
          elif [ "$statinst" = 'starting' ]; then
            stat="$FAILED"
          else
            :
          fi
        else
          :
        fi

        if [[ "$sysid" != '' && "$statmon" != "$stat" ]]; then
          echo "Resource ${name} status ${statmon} changed to ${stat}"

          if [ "$SNUPDATE" = 'table' ]; then
            SNURL="https://${NOW_HOST}/api/now/table"
            table='u_middleware_instanz'

# Returns the new status.
# {"result":{"name":"jvm.ope.as12345.1","u_monitor_status":"offline"}}
# Try to use the "script" above if an ACL error returned.
# {"error":{"detail":"ACL Exception Update Failed due to security constraints","message":"Operation Failed"},"status":"failure"}
            data='"'"${SNURL}/${table}/${sysid}"
            data=$data'?sysparm_display_value=false'
            data=$data'&sysparm_exclude_reference_link=true'
            data=$data'&sysparm_fields=name%2Cu_monitor_status"'
            data=$data' --request PUT --header "Accept:application/json"'
            data=$data' --header "Content-Type:application/json"'
            data=$data' --data "{\"u_monitor_status\":\"'"${stat}"'\"}"'
            data=$data' --insecure --user "'${SNUSER}'":"'${SNPASS}'"'
            data=$data' --silent --show-error'
          fi

# ServiceNow call.
          if [ "$SNDEBUG" != 'curl' ]; then
            ${SNCMD} ${data} >/dev/null 2>&1
            rc=$(($rc + $?))
          else
            echo ${SNCMD} ${data} >> ${SNLOG} 2>&1
#            result=`${SNCMD} ${data} | tee -a ${SNLOG}`
            ${SNCMD} ${data} >> ${SNLOG} 2>&1
            rc=$(($rc + $?))
            echo "" >> ${SNLOG}
          fi
        else
          if [ "$sysid" = '' ]; then
            echo "Resource ${name} not found"
          else
            echo "Resource ${name} status ${statmon} not changed"
          fi
        fi
      fi
    fi
  done
  RC="$rc"
elif [[ "$text" = *' reported:  Action '* ]]; then
  :
else
  set -A parms "$@"
# Used Command Parameters.
# EM Name.
#  em_name="${parms[0]}"
# EM Host.
#  em_host="${parms[1]}"
# EM IP Address.
#  em_addr="${parms[2]}"
# Alert Name.
  alert_name="${parms[0]}"
# Alert Status.
  alert_stat="${parms[1]}"
# Alert Time.
  alert_time="${parms[2]}"
# Use Agent Name(s) or Metric Name(s), not both.
# Agent Name(s).
#  agent_name="${parms[6]}"
  line="${parms[3]}"
# Metric Name(s).
#  metric_name="${parms[7]}"
  line="${parms[3]}"
# Previous Alert State.
  alert_prev="${parms[4]}"
# Has Alert State Changed.
#  alert_change="${parms[9]}"
# Threshold Value.
  alert_thres="${parms[5]}"

  alert=`echo ${alert_name} | sed -e 's/"//g'`
  time=`echo ${alert_time} | sed -e 's/"//g'`
# The normal state threshold is a blank.
  thres=`echo ${alert_thres} | sed -e 's/[ "]//g'`

# Map some values from APM to ServiceNow.
# Danger=3, Caution=2, Normal=1, Not reporting=0, In downtime period<0.
  case "$alert_stat" in
    '"3"') stat="$OFFLINE"; alert_stat='danger' ;; 
    '"-3"') stat="$ONLINE"; alert_stat='danger' ;; 
    '"danger"') stat="$OFFLINE"; alert_stat='danger' ;; 
    '"2"') stat="$OFFLINE"; alert_stat='caution' ;;
    '"-2"') stat="$ONLINE"; alert_stat='caution' ;;
    '"caution"') stat="$OFFLINE"; alert_stat='caution' ;;
    '"1"') stat="$ONLINE"; alert_stat='normal' ;;
    '"-1"') stat="$ONLINE"; alert_stat='normal' ;;
    '"normal"') stat="$ONLINE"; alert_stat='normal' ;;
    '"0"') stat="$UNKNOWN"; alert_stat='unknown' ;;
    *) stat="$UNKNOWN"; alert_stat='unknown' ;;
  esac

  echo "$line" | \
  awk '{
    l=split(substr($0, 2, length($0)-2), line, "\" \"")
    for(i=1; i<=l; i++) {
      if (match(line[i], "[[]") != 0) {
        line[i]=substr(line[i], 2, length(line[i])-2)
        v=split(line[i], val, ", ")
        for(j=1; j<=v; j++) {
          print val[j]
        }
      } else {
        print line[i]
      }
    }
  }' |  \
  while read -r line; do
    host=`echo "$line" | cut -f 2 -d '|'`
    if [[ "$host" = 'Custom Metric Host'* ]]; then
      host=`hostname`
    fi

# Handle status notifications only.
# Does not work well, use "Pass Text to Shell" instead.
    if [ "$SNLBL" = "CONNECTIONSTATUS" ]; then
      table='u_middleware_instanz'
      host=`echo "$line" | cut -f 6 -d '|'`
      agent=`echo "$line" | cut -f 8 -d '|'`
      attr=`echo "$agent" | cut -f 2 -d ':' | cut -f 1 -d ' ' | tr 'a-z' 'A-Z'`
# The metrics value is not available, use the threshold instead.
      [ -n "$alert_thres" ] && value="$alert_thres" || value=''
      agent=`echo "$agent" | cut -f 1 -d ':'`

      if [ "$SNLBL" = "$attr" ]; then
# Connection metrics have these values.
# No data=0, Connected=1, Slow report=2, Disconnected=3
        if [ "$value" -eq 1 -o "$value" -eq 2 ]; then
          stat="$ONLINE"
        else
          stat="$OFFLINE"
        fi
# Not a ConnectionStatus metric.
      else
        table=''
        echo "#### $line" >> ${SNLOG}
      fi

# Map the used resource name.
# Agent "env.reg.ope.as12345.1" to resource "jvm.ope.as12345.1".
      name="jvm."`echo "$agent" | cut -f 3-5 -d '.'`
    else
      table=''
      agent=`echo "$line" | cut -f 4 -d '|'`

      echo "#### $line" >> ${SNLOG}
    fi        

    host=`echo ${host%%.*} | tr 'a-z' 'A-Z'`

# Add the defined threshold value if available.
    if [ -n "$thres" ]; then
      line="${line} <> ${thres}"
    fi

#    text=`echo "APM ${agent} ${alert} in ${alert_stat} state" | sed -e 's/\&/\&amp;/g' \
#      -e 's/"//g' -e "s/'/\&apos;/g" -e 's/</\&lt;/g' -e 's/>/\&gt;/g'`
#    desc=`echo "${line}" | sed -e 's/\&/\&amp;/g' -e 's/"/\&quot;/g' \
#      -e "s/'/\&apos;/g" -e 's/</\&lt;/g' -e 's/>/\&gt;/g' \
#      -n -e 'H;${x;s/\n/\&#13;\&#10;/g;s/^\&#13;\&#10;//;p;}'`

#    alert=`echo ${alert} | sed -e 's/\&/\&amp;/g' -e 's/"/\&quot;/g' \
#      -e "s/'/\&apos;/g" -e 's/</\&lt;/g' -e 's/>/\&gt;/g'`

    if [ -n "$table" ]; then
      SNURL="https://${NOW_HOST}/api/now/table"
      table='u_middleware_instanz'

# Get the used sys_id.
# {"result":[{"sys_id":"ca83c3816f054b0027e0cc60cb1f4e8e"}]} or
# {"result":[{"sys_id":"996c61a16fff3280029ecc60cb1f4ed8","u_instanz_status":"up","u_monitor_status":"1001"}]}
# {"result":[{"sys_id":"996c61a16fff3280029ecc60cb1f4ed8","u_monitor_status":"1001"}]}
# {"result":[]}
      data='"'"${SNURL}/${table}"
      data=$data'?sysparm_query=name='"${name}"
      data=$data'&sysparm_fields=sys_id%2Cu_instanz_status%2Cu_monitor_status'
      data=$data'&sysparm_limit=1"'
      data=$data' --request GET'
      data=$data' --insecure --user "'${SNUSER}'":"'${SNPASS}'"'
      data=$data' --silent --show-error'

      if [ "$SNDEBUG" != 'curl' ]; then
        result=`${SNCMD} ${data} 2>/dev/null`
      else
        date >> ${SNLOG}
        echo ${SNCMD} ${data} >> ${SNLOG} 2>&1
        result=`${SNCMD} ${data} | tee -a ${SNLOG}`
        echo "" >> ${SNLOG}
      fi

#      sysid=`echo "$result" | sed -n -e 's/.*{"sys_id":"\(.*\)"}.*/\1/p'`
      sysid=`echo "$result" | sed -n -e 's/.*"sys_id":"\([^"]*\)".*/\1/p'`
      statinst=`echo "$result" | sed -n -e 's/.*"u_instanz_status":"\([^"]*\)".*/\1/p'`
      statmon=`echo "$result" | sed -n -e 's/.*"u_monitor_status":"\([^"]*\)".*/\1/p'`

# Status mapping.
      if [ "$stat" = "$ONLINE" ]; then
        if [ "$statinst" = 'control' ]; then
          : 
        elif [ "$statinst" = 'down' ]; then
          stat="$STUCK"
        elif [ "$statinst" = 'shutdown' ]; then
          stat="$STUCK"
        elif [ "$statinst" = 'stopping' ]; then
          stat="$STUCK"
        else
          :
        fi
      elif [ "$stat" = "$OFFLINE" ]; then
        if [ "$statinst" = 'control' ]; then
          : 
        elif [ "$statinst" = 'up' ]; then
          stat="$FAILED"
        elif [ "$statinst" = 'starting' ]; then
          stat="$FAILED"
        else
          :
        fi
      else
        :
      fi

      if [[ "$sysid" != '' && "$statmon" != "$stat" ]]; then
        echo "Resource ${name} status ${statmon} changed to ${stat}"

        if [ "$SNUPDATE" = 'table' ]; then
          SNURL="https://${NOW_HOST}/api/now/table"
          table='u_middleware_instanz'

# Returns the new status.
# {"result":{"name":"jvm.ope.as12345.1","u_monitor_status":"offline"}}
# Try to use the "script" above if an ACL error returned.
# {"error":{"detail":"ACL Exception Update Failed due to security constraints","message":"Operation Failed"},"status":"failure"}
          data='"'"${SNURL}/${table}/${sysid}"
          data=$data'?sysparm_display_value=false'
          data=$data'&sysparm_exclude_reference_link=true'
          data=$data'&sysparm_fields=name%2Cu_monitor_status"'
          data=$data' --request PUT --header "Accept:application/json"'
          data=$data' --header "Content-Type:application/json"'
          data=$data' --data "{\"u_monitor_status\":\"'"${stat}"'\"}"'
          data=$data' --insecure --user "'${SNUSER}'":"'${SNPASS}'"'
          data=$data' --silent --show-error'
        fi

# ServiceNow call.
        if [ "$SNDEBUG" != 'curl' ]; then
          ${SNCMD} ${data} >/dev/null 2>&1
          rc=$(($rc + $?))
        else
          echo ${SNCMD} ${data} >> ${SNLOG} 2>&1
#          result=`${SNCMD} ${data} | tee -a ${SNLOG}`
          ${SNCMD} ${data} >> ${SNLOG} 2>&1
          rc=$(($rc + $?))
          echo "" >> ${SNLOG}
        fi 
      else
        if [ "$sysid" = '' ]; then
          echo "Resource ${name} not found"
        else
          echo "Resource ${name} status ${statmon} not changed"
        fi
      fi
    fi
  done
  RC="$rc"
fi

exit $RC
