#!/bin/ksh
#set -vx

# Introscope Enterprise Manager ZIS alarm script.

# Enable "Notify by individual metrix" for the used alert if you find
# some messages like "Action "ZIS Notification" failed to execute
# shell command "./scripts/zexec.sh"" in the log. In debug mode you find  
# errors like "The parameter or environment lists are too long.".

# The action Shell Command parameter is the way how to handle the alert and
# some of the following parameter "Pass Text to Shell" or
# "Alert Name", "Alert Status", "Alert Time" and "Agent Name(s)" or
# "Metric Name(s)". And additional "Previous Alert State" and
# "Threshold Value".
# Not used are "EM Name", "EM Host", "EM IP Address" and
# "Has Alert State Changed".

if [ -L $0 ]; then
  ls="`ls -l $0`"
  PRG="${ls##* -> }"
else
  PRG="$0"
fi

if [ "$PRG" = ^/ ]; then
  DIR=`dirname $PRG`
else
  DIR="`cd \`dirname $PRG\`; pwd -P`"
  PRG="`cd \`dirname $PRG\`; pwd -P`/`basename $PRG`"
fi

DESC="Introscope Enterprise Manager alarm script"
SCRIPT=${PRG##*/}
NAME=${SCRIPT%.*}

DIENST=`echo "$DIR" | cut -c6-12`
OS=`uname -s | tr 'A-Z' 'a-z'`

cd $DIR

# Monitor resource status.
UNKNOWN=0
ONLINE=1
OFFLINE=2
FAILED=3
STUCK=4
#PENONLINE=5
#PENOFFLINE=6
MIXED=7
INELIGIBLE=8

# ZIS init.
ZEXCMD="./zexec.pl"
ZEXCFG="./zexec.conf"
ZEXLOG="../logs/zexec.log"
ZEXLBL="INTROSCOPE_MAIN"
ZEXCLS="INTROSCOPE"
ZEXEVU="INTROSCOPE"
ZEXSRC="INTROSCOPE"
#ZEXMSG="INTR000I"
ZEXMSG="INTR001I"
ZEXSTA="Info"

# ZIS ZExec return code handling.
RC=0
rc=0

# Handle the command parameters.
USERLBL=''
USERCLS=''
USERMSG=''
USERSTA=''
parm=`echo "$1" | cut -f 1 -d ' '`
if [ -n "$parm" ]; then
  ZEXLBL="$parm"
  USERLBL="$parm"
fi
parm=`echo "$1" | cut -f 2 -d ' '`
if [ -n "$parm" ]; then
  ZEXCLS="$parm"
  USERCLS="$parm"
fi
parm=`echo "$1" | cut -f 3 -d ' '`
if [ -n "$parm" ]; then
#  ZEXEVU="$parm"
  ZEXMSG="$parm"
  USERMSG="$parm"
fi
parm=`echo "$1" | cut -f 4 -d ' '`
if [ -n "$parm" ]; then
  ZEXSTA="$parm"
  USERSTA="$parm"
fi
shift

if [ -z "$USER" ]; then
  USER=`whoami 2>/dev/null`
  if [ -z "$USER" ]; then
    USER="$LOGNAME"
  fi
fi
user=`echo "$USER" | tr 'a-z' 'A-Z'`

if [[ "$DIENST" = 'as'* ]]; then
  dienst="$DIENST"
else
  [ -n "$USER" ] && dienst="$USER" || dienst="$LOGNAME"
fi

if [ -z "$HOST" ]; then
  HOST=`hostname`
fi
host=`echo ${HOST%%.*} | tr 'a-z' 'A-Z'`

#echo "$*" >> ${ZEXLOG} 

# Handle ZExec events like the EMail alerts send by Introscope.
# from: introscope@f-i.de
# subject: CA APM Alert from ${EM_Host}: ${Alert_Name} in ${Alert_State} state
# body: Alert Name: ${Alert_Name}
# Time Triggered: ${Alert_Time}
# Alert Status: ${Alert_State}
# Problem Details:
# ${Problem_Detail_List}

desc='Introscope alert.'

# Handle the text input line.
text="$*"
alert_stat='Undef'

if [[ "$text" = *' reported:  The Alert '* ]]; then
# Used Command Parameters.
# Pass Text to Shell.
  echo "$text" | \
  awk '{
    if (index($0, "reported:  The Alert") != 0) {
      l=split($0, line, "         ")
      for(i=1; i<=l; i++) {
        if (i == 1) {
          h=split(line[i], head, ":  ")
          print head[1]
          print substr(head[2], 1, length(head[2])-1)
        } else {
          print line[i]
        }
      }
    }
  }' | \
  while read -r line; do
    if [[ "$line" = *'Introscope Enterprise Manager'* ]]; then
      time=`echo "$line" | cut -f 1-4 -d ' '`
    elif [[ "$line" = *'The Alert'*'opened a caution alert'* ]]; then
      alert=`echo ${line} | sed -n -e 's/.*The Alert \(.*\) opened a caution alert.*/\1/p'`
      stat='Warning'
      alert_stat='caution'
    elif [[ "$line" = *'The Alert'*'exceeded caution target'* ]]; then
      alert=`echo ${line} | sed -n -e 's/.*The Alert \(.*\) exceeded caution target.*/\1/p'`
      stat='Warning'
      alert_stat='caution'
    elif [[ "$line" = *'The Alert'*'opened a danger alert'* ]]; then
      alert=`echo ${line} | sed -n -e 's/.*The Alert \(.*\) opened a danger alert.*/\1/p'`
      stat='Error'
      alert_stat='danger'
    elif [[ "$line" = *'The Alert'*'exceeded danger target'* ]]; then
      alert=`echo ${line} | sed -n -e 's/.*The Alert \(.*\) exceeded danger target.*/\1/p'`
      stat='Error'
      alert_stat='danger'
    elif [[ "$line" = *'The Alert'*'closed a caution alert'* ]]; then
      alert=`echo ${line} | sed -n -e 's/.*The Alert \(.*\) closed a caution alert.*/\1/p'`
      stat='OK'
      alert_stat='normal'
    elif [[ "$line" = *'The Alert'*'closed a danger alert'* ]]; then
      alert=`echo ${line} | sed -n -e 's/.*The Alert \(.*\) closed a danger alert.*/\1/p'`
      stat='OK'
      alert_stat='normal'
    else
      host=`echo "$line" | cut -f 2 -d '|'`
      if [[ "$host" = 'Custom Metric Host'* ]]; then
        host=`hostname`
      fi
      host=`echo ${host%%.*} | tr 'a-z' 'A-Z'`
      agent=`echo "$line" | cut -f 4 -d '|'`

      text=`echo "APM ${agent} ${alert} in ${alert_stat} state" | sed -e 's/\&/\&amp;/g' \
        -e 's/"//g' -e "s/'/\&apos;/g" -e 's/</\&lt;/g' -e 's/>/\&gt;/g'`
      desc=`echo ${line} | sed -e 's/\&/\&amp;/g' -e 's/"/\&quot;/g' \
        -e "s/'/\&apos;/g" -e 's/</\&lt;/g' -e 's/>/\&gt;/g'`

      alert=`echo ${alert} | sed -e 's/\&/\&amp;/g' -e 's/"/\&quot;/g' \
        -e "s/'/\&apos;/g" -e 's/</\&lt;/g' -e 's/>/\&gt;/g'`

      data='<?xml version="1.0" encoding="ISO-8859-1"?>'
      data=$data"<event>"
      data=$data"<klasse>$ZEXCLS</klasse>"
#      data=$data"<eventunit>$ZEXEVU</eventunit>"
      data=$data"<quelle>$ZEXSRC</quelle>"
      data=$data"<melder>${host}</melder>"
      data=$data"<zeitpunkt>${time}</zeitpunkt>"
#      data=$data"<ev_id>${evid}</ev_id>"
      data=$data"<ev_id>${agent}_${alert}</ev_id>"
      data=$data"<msgid>$ZEXMSG</msgid>"
      data=$data"<status>${stat}</status>"
      data=$data"<meldung>${text}</meldung>"
      data=$data"<beschreibung>${desc}</beschreibung>"
      data=$data"</event>"

# ZIS ZExec call.
      echo ${ZEXCMD} -c "${ZEXCFG}" -l "${ZEXLBL} ${data}" >> ${ZEXLOG} 2>&1
      ${ZEXCMD} -c "${ZEXCFG}" -l "${ZEXLBL} ${data}" >> ${ZEXLOG} 2>&1
      rc=$(($rc + $?))
    fi
  done
  RC="$rc"
elif [[ "$text" = *' reported:  Action '* ]]; then
  :
else
  set -A parms "$@"
# Used Command Parameters.
# EM Name.
#  em_name="${parms[0]}"
# EM Host.
#  em_host="${parms[1]}"
# EM IP Address.
#  em_addr="${parms[2]}"
# Alert Name.
  alert_name="${parms[0]}"
# Alert Status.
  alert_stat="${parms[1]}"
# Alert Time.
  alert_time="${parms[2]}"
# Use Agent Name(s) or Metric Name(s), not both.
# Agent Name(s).
#  agent_name="${parms[6]}"
  line="${parms[3]}"
# Metric Name(s).
#  metric_name="${parms[7]}"
  line="${parms[3]}"
# Previous Alert State.
  alert_prev="${parms[4]}"
# Has Alert State Changed.
#  alert_change="${parms[9]}"
# Threshold Value.
  alert_thres="${parms[5]}"

  alert=`echo ${alert_name} | sed -e 's/"//g'`
  time=`echo ${alert_time} | sed -e 's/"//g'`
# The normal state threshold is a blank.
  thres=`echo ${alert_thres} | sed -e 's/[ "]//g'`

# Map some values from APM to ZIS.
# Danger=3, Caution=2, Normal=1, Not reporting=0, In downtime period<0.
  case "$alert_stat" in
    '"3"') stat='Error'; alert_stat='danger' ;; 
    '"-3"') stat='OK'; alert_stat='danger' ;; 
    '"danger"') stat='Error'; alert_stat='danger' ;; 
    '"2"') stat='Warning'; alert_stat='caution' ;;
    '"-2"') stat='OK'; alert_stat='caution' ;;
    '"caution"') stat='Warning'; alert_stat='caution' ;;
    '"1"') stat='OK'; alert_stat='normal' ;;
    '"-1"') stat='OK'; alert_stat='normal' ;;
    '"normal"') stat='OK'; alert_stat='normal' ;;
    '"0"') stat='Undef'; alert_stat='unknown' ;;
    *) stat='Undef'; alert_stat='unknown' ;;
  esac

  echo "$line" | \
  awk '{
    l=split(substr($0, 2, length($0)-2), line, "\" \"")
    for(i=1; i<=l; i++) {
      if (match(line[i], "[[]") != 0) {
        line[i]=substr(line[i], 2, length(line[i])-2)
        v=split(line[i], val, ", ")
        for(j=1; j<=v; j++) {
          print val[j]
        }
      } else {
        print line[i]
      }
    }
  }' |  \
  while read -r line; do
    host=`echo "$line" | cut -f 2 -d '|'`
    if [[ "$host" = 'Custom Metric Host'* ]]; then
      host=`hostname`
    fi
    host=`echo ${host%%.*} | tr 'a-z' 'A-Z'`
    agent=`echo "$line" | cut -f 4 -d '|'`

# Add the defined threshold value if available.
    if [ -n "$thres" ]; then
      line="${line} <> ${thres}"
    fi

    text=`echo "APM ${agent} ${alert} in ${alert_stat} state" | sed -e 's/\&/\&amp;/g' \
      -e 's/"//g' -e "s/'/\&apos;/g" -e 's/</\&lt;/g' -e 's/>/\&gt;/g'`
    desc=`echo "${line}" | sed -e 's/\&/\&amp;/g' -e 's/"/\&quot;/g' \
      -e "s/'/\&apos;/g" -e 's/</\&lt;/g' -e 's/>/\&gt;/g' \
      -n -e 'H;${x;s/\n/\&#13;\&#10;/g;s/^\&#13;\&#10;//;p;}'`

    alert=`echo ${alert} | sed -e 's/\&/\&amp;/g' -e 's/"/\&quot;/g' \
      -e "s/'/\&apos;/g" -e 's/</\&lt;/g' -e 's/>/\&gt;/g'`

    data='<?xml version="1.0" encoding="ISO-8859-1"?>'
    data=$data"<event>"
    data=$data"<klasse>$ZEXCLS</klasse>"
#    data=$data"<eventunit>$ZEXEVU</eventunit>"
    data=$data"<quelle>$ZEXSRC</quelle>"
    data=$data"<melder>${host}</melder>"
    data=$data"<zeitpunkt>${time}</zeitpunkt>"
#    data=$data"<ev_id>${evid}</ev_id>"
    data=$data"<ev_id>${agent}_${alert}</ev_id>"
    data=$data"<msgid>$ZEXMSG</msgid>"
    data=$data"<status>${stat}</status>"
    data=$data"<meldung>${text}</meldung>"
    data=$data"<beschreibung>${desc}</beschreibung>"
    data=$data"</event>"

# ZIS ZExec call.
    echo ${ZEXCMD} -c "${ZEXCFG}" -l "${ZEXLBL} ${data}" >> ${ZEXLOG} 2>&1
    ${ZEXCMD} -c "${ZEXCFG}" -l "${ZEXLBL} ${data}" >> ${ZEXLOG} 2>&1
    rc=$(($rc + $?))
  done
  RC="$rc"
fi

exit $RC
